PRINTLIB5_HEADER = \
    'D E C A Y    R A D I A T I O N    D I S C R E T E   S P E C T R A'

PRINTLIB4_HEADER = \
    'C O L L A P S E D    C R O S S     S E C T I O N S     I N     B A R N S'

PRINTLIB4_START_HEADER = \
    'parent reaction  daughter cross section   +-error            parent reaction  daughter cross section   +-error'

PRINTLIB4_END_HEADER = \
    'N E U T R O N      S P E C T R U M'